import { basename, resolve } from 'node:path';
import { NSedError } from './models.js';
import { showOutput } from './output.js';
export async function parseCommand(type1, command1) {
    // Parse the command
    if (type1 === 'function') {
        try {
            let commandFunction1 = await import(resolve(process.cwd(), command1));
            if (commandFunction1.default) {
                // CJS/MJS inteoperability
                commandFunction1 = commandFunction1.default;
            }
            if (typeof commandFunction1 !== 'function') {
                throw new NSedError(`File "${command1}" must export a function.`);
            }
            return {
                type: 'function',
                command: commandFunction1
            };
        } catch (error1) {
            if (error1.code === 'ENSED') {
                throw error1;
            }
            throw new NSedError(`Cannot require file "${command1}".`);
        }
    } else if (command1.startsWith('.') || command1.startsWith('[')) {
        command1 = `$data${command1}`;
    } else if (command1.startsWith('/')) {
        command1 = `$data.toString().match(${command1})`;
    } else if (/^\$\d+$/.test(command1)) {
        command1 = `RegExp.${command1}`;
    }
    return {
        type: type1,
        command: command1
    };
}
export async function requireModule(modulePath1) {
    // Camelcase the module
    const moduleName1 = basename(modulePath1).toLowerCase().replace(/[./-_]([\da-z])/, (_1, t1)=>t1.toUpperCase());
    try {
        Object.defineProperty(globalThis, moduleName1, {
            value: await import(modulePath1)
        });
        resolve();
    } catch  {
        throw new NSedError(`Cannot find module "${modulePath1}".`);
    }
}
export async function executeCommands($data, $index, commands) {
    // Apply all commands
    for (const cmd of commands){
        const { type, command } = cmd;
        try {
            // eslint-disable-next-line no-eval
            const res = await (type === 'function' ? command($data, $index) : eval(command));
            // Update $data
            switch(type){
                case 'filter':
                    // A filter returned a falsy value, filter out
                    if (!res) {
                        return;
                    }
                    break;
                case 'reverseFilter':
                    // A reverse filter returned a truthy value, filter out
                    if (res) {
                        return;
                    }
                    break;
                default:
                    $data = res;
            }
        } catch (error) {
            throw new NSedError(`Invalid command "${command}": [${error.name}] ${error.message}.`);
        }
    }
    showOutput($data);
}
